@@ -18,7 +18,7 @@ import shutil
 
 from pytodotxt import TodoTxt, Task, version
 
-from cursedspace import Application, Panel, ScrollPanel, InputLine, ShellContext
+from cursedspace import Application, Panel, ScrollPanel, InputLine, ShellContext, Completion
 
 from pter import common
 from pter import utils
@@ -201,7 +201,7 @@ class HelpBar(Panel):
             actions = ['cancel', 'select-item']
         elif isinstance(self.app.focus[-1], TaskCreator):
             actions = ['cancel', 'submit-input']
-            if len(self.app.sources) > 0:
+            if len(self.app.sources) > 1:
                 actions += ['select-file']
             mapping = self.app.editor_key_mapping
         elif isinstance(self.app.focus[-1], TaskEditor):
@@ -433,19 +433,22 @@ class TaskList(ScrollPanel):
                 elem = elem + extra_right
 
             elemlen = len(elem)
-            if elemlen >= maxwidth:
+            if elemlen > maxwidth:
                 cut_off = True
                 elem = elem[:maxwidth]
 
-            self.win.addstr(y, x, elem, self.app.color(element.color, is_selected, baseattrs))
+            try:
+                self.win.addstr(y, x, elem, self.app.color(element.color, is_selected, baseattrs))
+            except curses.error:
+                pass
 
             if cut_off:
                 attrs = common.SETTING_COL_OVERFLOW
                 try:
-                    self.win.addstr(y, x+maxwidth+1-len(self.app.overflow_marker[1]),
+                    self.win.addstr(y, x+maxwidth-len(self.app.overflow_marker[1]),
                                     self.app.overflow_marker[1],
                                     self.app.color(attrs, is_selected, baseattrs))
-                except:
+                except curses.error:
                     pass
             return elem
 
@@ -517,8 +520,8 @@ class TaskList(ScrollPanel):
 
         if x < maxwidth:
             try:
-                self.win.addstr(y, x, ' '*(maxwidth-x+1), self.app.color(baseattrs, is_selected))
-            except:
+                self.win.addstr(y, x, ' '*(maxwidth-x), self.app.color(baseattrs, is_selected))
+            except curses.error:
                 pass
 
     def jump_to(self, item):
@@ -569,7 +572,7 @@ class RemappedInputLine(InputLine):
         pass
 
     def on_change(self):
-        pass
+        self.update_completion()
 
     def on_timeout(self):
         return False
@@ -583,6 +586,8 @@ class RemappedInputLine(InputLine):
 
         if key == Key.TIMEOUT:
             must_repaint = self.on_timeout()
+        elif self.completion is not None and self.completion.handle_key(key):
+            handled = True
         elif fnc == 'cancel':
             self.on_cancel()
         elif fnc == 'submit-input':
@@ -626,10 +631,47 @@ class RemappedInputLine(InputLine):
         return handled
 
 
+class ContextCompletion(Completion):
+    def close(self):
+        retval = super().close()
+        if retval:
+            self.app.paint()
+        return retval
+
+    def update(self, y, x):
+        span = self.inputline.current_word()
+
+        if span is None:
+            self.close()
+            return
+
+        word = self.inputline.text[span[0]:self.inputline.cursor]
+        if word.startswith('@'):
+            options = {context
+                       for context in sum([list(source.contexts) for source in self.app.sources], start=[])
+                       if context.startswith(word)}
+        elif word.startswith('+'):
+            options = {project
+                       for project in sum([list(source.projects) for source in self.app.sources], start=[])
+                       if project.startswith(word)}
+        else:
+            return
+
+        if len(options) == 0 or (len(options) == 1 and word in options):
+            self.close()
+            return
+
+        options = list(options)
+        options.sort()
+        self.set_alternatives(options, (y, x))
+        self.app.paint()
+
+
 class SearchBar(RemappedInputLine):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.previous_text = self.text
+        self.completion = ContextCompletion(self)
 
     def paint(self, clear=False):
         self.border = Panel.BORDER_NONE
@@ -652,7 +694,11 @@ class SearchBar(RemappedInputLine):
         self.win.addstr(0, 0, visible_text, attr)
         self.win.noutrefresh()
 
+        if self.completion is not None:
+            self.completion.paint(clear)
+
     def on_change(self):
+        self.update_completion()
         self.app.search.text = self.text
         self.app.search.parse()
         self.app.apply_search()
@@ -669,6 +715,7 @@ class SearchBar(RemappedInputLine):
         self.unfocus()
 
     def unfocus(self):
+        self.cursor_changed = False
         self.app.show_cursor(False)
         self.app.focus.pop(-1)
         self.scroll()
@@ -729,10 +776,12 @@ class UserInput(Panel):
     def on_cancel(self):
         if self.custom_on_cancel is not None:
             self.custom_on_cancel()
+        self.app.show_cursor(False)
         self.destroy()
 
     def on_submit(self):
         self.on_accept(self.editor.text)
+        self.app.show_cursor(False)
         self.destroy()
 
 
@@ -742,6 +791,7 @@ class TaskEditor(UserInput):
         self.task = task
         self.on_accept = self.save_changes
         self.on_cancel = lambda: self.app.show_cursor(False)
+        self.editor.completion = ContextCompletion(self.editor)
 
     def save_changes(self, text):
         self.app.show_cursor(False)
@@ -786,7 +836,7 @@ class TaskCreator(TaskEditor):
     def handle_key(self, key):
         fnc = self.app.editor_key_mapping.get(str(key), None)
 
-        if fnc == 'select-file':
+        if fnc == 'select-file' and len(self.app.sources) > 1:
             self.app.show_cursor(False)
             self.app.focus.append(SourceSelector(self,
                                                  self.app.sources,
@@ -832,7 +882,7 @@ class TaskCreator(TaskEditor):
     def paint(self, clear=False):
         super().paint(clear)
 
-        if len(self.app.sources) > 0:
+        if len(self.app.sources) > 1:
             label = self.task.todotxt.displayname
             self.win.addstr(2, self.dim[1]-len(label)-3, f"┤{label}├")
             self.win.noutrefresh()
@@ -863,8 +913,11 @@ class Selector(RemappedScrollPanel):
         if hasattr(self.parent, 'autoresize'):
             self.parent.autoresize()
         maxheight, maxwidth = self.app.size()
-        w = maxwidth//4 if maxwidth > 11 else maxwidth
-        h = min(maxheight//2 if maxheight > 9 else maxheight, len(self.items)+2)
+        labelwidth = max([len(self.make_label(item)) for item in self.items]) \
+                   + len(self.indicator(True)) \
+                   + len(str(len(self.items)))
+        w = min(2*maxwidth//3 if maxwidth > 11 else maxwidth, labelwidth)
+        h = min(4*maxheight//5 if maxheight > 9 else maxheight, len(self.items)+2)
         self.resize(h, w)
         self.move((maxheight-h)//2, (maxwidth-w)//2)
 
@@ -912,10 +965,13 @@ class Selector(RemappedScrollPanel):
             pointer = " "*len(pointer)
         return pointer
 
+    def make_label(self, item):
+        return str(item)
+
     def do_paint_item(self, y, x, maxwidth, is_selected, item):
         attrs = self.app.color(common.SETTING_COL_NORMAL, is_selected)
 
-        label = self.indicator(is_selected) + self.number_prefix(item) + item
+        label = self.indicator(is_selected) + self.number_prefix(item) + self.make_label(item)
 
         self.win.addstr(y, x, label[:maxwidth], attrs)
     
@@ -930,10 +986,8 @@ class SourceSelector(Selector):
         super().__init__(*args, **kwargs)
         self.title = 'Select source'
 
-    def do_paint_item(self, y, x, maxwidth, is_selected, item):
-        attrs = self.app.color(common.SETTING_COL_NORMAL, is_selected)
-        label = self.indicator(is_selected) + self.number_prefix(item) + item.displayname
-        self.win.addstr(y, x, label[:maxwidth], attrs)
+    def make_label(self, item):
+        return item.displayname
 
 
 class HelpScreen(RemappedScrollPanel):
@@ -1303,6 +1357,7 @@ class CursesApplication(Application):
             self.screen.addstr(1, 0, "─"*self.size()[1], attr)
         except:
             pass
+        self.screen.noutrefresh()
 
         self.tasks.paint()
         self.search_bar.paint()
