@@ -434,6 +434,12 @@ def update_displaynames(sources):
         pos += 1
 
 
+def delegate_task(task, marker):
+    if task.description is None:
+        task.description = ''
+    task.description += ' ' + marker
+
+
 def execute_delegate_action(task, to_attr, marker, action):
     if action == common.DELEGATE_ACTION_NONE:
         return
